
//importation des bibliothèques du capteur de Co2 (https://wiki.seeedstudio.com/Grove-CO2_Temperature_Humidity_Sensor-SCD30/) et de la carte SD (https://wiki.seeedstudio.com/SD_Card_shield_V4.0/)

#include <SD.h>       // importation bibliothèque de la carte SD 
#include <SPI.h>
#include "SCD30.h"    // importation bibliothèque du capteur de CO2
#include <rgb_lcd.h> // importation de la bibliothèque de l'écran LCD

rgb_lcd rgbLcd;   // déclaration de l'afficheur
File monFichier;  // déclaration du fichier de la carte SD 

// Ligne de code pour communiquer avec le capteur CO2
#if defined(ARDUINO_ARCH_AVR)
    #pragma message("Defined architecture for ARDUINO_ARCH_AVR.")
    #define SERIAL Serial
#elif defined(ARDUINO_ARCH_SAM)
    #pragma message("Defined architecture for ARDUINO_ARCH_SAM.")
    #define SERIAL SerialUSB
#elif defined(ARDUINO_ARCH_SAMD)
    #pragma message("Defined architecture for ARDUINO_ARCH_SAMD.")  
    #define SERIAL SerialUSB
#elif defined(ARDUINO_ARCH_STM32F4)
    #pragma message("Defined architecture for ARDUINO_ARCH_STM32F4.")
    #define SERIAL SerialUSB
#else
    #pragma message("Not found any architecture.")
    #define SERIAL Serial
#endif

void setup()
{
Serial.begin(9600);// attention régler les baud dans le moniteur série 
// initialisation de l'écran LCD
   rgbLcd.begin(16,2);

//initialisation carte SD 
  Serial.print("initialisation de la carte SD ...");
  if(!SD.begin(4)){
    Serial.println("l initialisation de la carte a echoue !");
    return;
  }
 Serial.println("l initialisation de la carte sd a reussi!");

//initialisation capteur
    Wire.begin();
    scd30.initialize();
    Serial.println("capteur est initialise");
    
//Ecriture sur le fichier de la carte SD de l'entête des grandeurs mesurées
 monFichier=SD.open("test1.txt",FILE_WRITE); // Donner un nom à votre fichier 
 if(monFichier){
     Serial.println("en train d ecrire ds fichier test1.txt");
     monFichier.print("temps  en s");
     monFichier.print("\t");  //cette ligne permet de réaliser une tabulation afin d'organiser les données dans le fichier 
     monFichier.println("taux de CO2 en ppm");
     monFichier.close();
     Serial.println("entete du tableau faite"); 
    }
    else{Serial.println("erreur lors de la l'ouverture du fichier");}
}

void loop()
{
 float result[3] = {0}; 
 
//réalisation des mesures
if(scd30.isAvailable())
   {
     scd30.getCarbonDioxideConcentration(result);
     unsigned long t=millis();       
          
    // Vérification si données bien reçues
     if (isnan(t) || isnan(result[0])) 
     {
     Serial.println(F("Aucune valeur retournée par le DHT22. Est-il bien branché ?"));
     delay(2000);
     return;         // Si aucune valeur n'a été reçue par l'Arduino, on attend 2 secondes, puis on redémarre la fonction loop()
     }
     //On affiche les mesures sur l'écran LCD 
        rgbLcd.setCursor(0, 0); // positionnement du curseur au début de la première ligne
        rgbLcd.print("CO2  en ppm ");
        rgbLcd.setCursor(0,1); // positionnement du curseur au début de la première ligne  
        rgbLcd.print(result[0]);
        
     //on écrit dans le fichier qui est sur la carte SD les mesures sous forme organisée 
      monFichier=SD.open("test1.txt",FILE_WRITE); // attention le nom du fichier doit être le même que sur la ligne 49
        if(monFichier)
        { 
        monFichier.print(t/1000);  // on écrit la durée en s 
        monFichier.print("\t");    // on sépare la donnée avec une tabulation (permettra de faire un copier coller )        
        monFichier.println(result[0]);
        monFichier.close();       // L'enregistrement des données se fait au moment de la clôture du fichier
        Serial.println(F("Enregistrement réussi en carte SD"));
        }
        else {
           Serial.println(F("Erreur lors de la tentative d'ouverture du fichier en écriture, sur la carte SD"));
             }

//Pour récupérer les données sur la carte SD, relier la carte SD à un ordinateur puis ouvrir le fichier avec le bloc note
//Faire un copier coller du fichier dans un tableur, attention il faudra remplacer les points des nombres décimaux par des virgules 

//Pour rendre le dispositif nomade : on téléverse le programme sur la carte Arduino, on alimente la carte avec une pile 9 V et on enlève le port USB, 
// En faisant un Reset sur la carte Arduino, on recommence une série de mesures qui sera dans le même fichier 
// Pour arrêter réenregistrement il suffit de débrancher la pile 
// Pour recommencer un enregistrement, il suffit de rebrancher la pile et faire un Resset 

// Le capteur permet aussi de mesurer la température en °C -result[1]- et le taux d'himidité en % -result[2]-
// Il faudra alors rajouter des lignes de code après les lignes 54 et 90. 

   } 
 delay(5000);
}
